--[[ 
    All shadows are first drawn onto a canvas, which is then drawn onto a stencil 
    that uses a sprite as a mask, so that the shadows only appear on the shape of
    the sprite. Shadows are blended together, as they would be if there were only a
    single lightsource.
  ]]

--RESOURCES/SETTINGS
love.graphics.setBackgroundColor (0,0.4,0)
love.graphics.setDefaultFilter ('nearest','nearest')

local blob = love.graphics.newImage ('images/blobTextured.png')
local blobMask = love.graphics.newImage ('images/blobMask.png')
local shadow = love.graphics.newImage ('images/shadow.png')

local shadowCanvas = love.graphics.newCanvas() --canvas on which to draw the shadows
local mx, my = love.mouse.getPosition()
--

--LOAD
function love.load ()
 love.graphics.setCanvas(shadowCanvas)
 love.graphics.clear(0,0,0)
 love.graphics.setCanvas()
end
--

--MASK SHADER
local mask_shader = love.graphics.newShader[[
   vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
      if (Texel(texture, texture_coords).rgb == vec3(0.0)) {
         // a discarded pixel wont be applied as the stencil.
         discard;
      }
      return vec4(1.0);
   }
]]
--

--SET STENCIL
local function myStencilFunction()
    
 love.graphics.setShader(mask_shader)
  love.graphics.draw(blobMask,100,100) --use this sprite as a mask
 love.graphics.setShader()
 
end
--

--DRAW STENCIL
local function drawStencil()
 love.graphics.stencil(myStencilFunction, "replace", 1) --1
 love.graphics.setStencilTest("greater", 0) --0
 
 love.graphics.setColor(1,1,1,0.4) --transparency/color of shadows
  love.graphics.draw(shadowCanvas) --draw shadow canvas
 love.graphics.setColor(1,1,1,1) --reset to white
  
 love.graphics.setStencilTest()
end
--

--UPDATE CANVAS
local function updateCanvas(canvas)
love.graphics.setCanvas{canvas,stencil=true}
 love.graphics.clear() --clear (white/normal)
 love.graphics.setBlendMode("alpha", "premultiplied") --darken
  love.graphics.draw (shadow,mx-48,my-48)--draw shadow at mouse x,y
  love.graphics.draw (shadow,100,100)--draw shadow
 love.graphics.setBlendMode("alpha") --reset blend mode to normal
 love.graphics.setCanvas()   
end
--

--UPDATE
function love.update ()

 mx, my = love.mouse.getPosition()
 updateCanvas(shadowCanvas)
 
end
--

--DRAW
function love.draw()

 love.graphics.draw (blob,100,100)
 drawStencil() --draw stencil using sprite as mask

end
--